<?php

namespace SM\Factory;

use Doctrine\ORM\EntityManager;
use Doctrine\DBAL\Connection;
use Doctrine\ORM\Query\Expr;
use Teknisa\Model\Traits\TQuery;

class RelatorioCdaFactory
{
    use TQuery;
    protected $connection;
    protected $entityManager;

    public function __construct(EntityManager $entityManager)
    {
        $this->entityManager = $entityManager;
        $this->connection = $entityManager->getConnection();
    }

    public function getRelatorioCda($params): array
    {
        try {
            $queryBuilder = $this->connection->createQueryBuilder();

            $queryBuilder
                ->select([
                    'FUNCIONARIO.NMFUNC AS NMOPERADORCDAREL',
                    'FUNCIONARIO.CDFUNC AS CDOPERADORCDAREL',
                    "TO_CHAR(SERVPREST.DTSERVPREST, 'DD/MM/YYYY') AS DTSERVPRESTCDAREL",
                    'SERVPREST.IDSERVPREST AS IDSERVPRESTCDAREL',
                    'CLIENTE.NMFANTCLIE AS NMFANTRAZCDAREL',
                    "NVL(SERVPREST.IDSITSERVPREST, 'P') AS IDSITSERVPRESTCDAREL",
                    'CLIENTE.NMFANTCLIE AS CLIENTECDAREL',
                    'CLIENTE.CDCLIENTE',
                    'CLIENTE.IDCLIEATIV',
                    'SERVPREST.CDINTECLIE',
                    'SERVPREST.NMINTCLIENTE AS NMINTECLIECDAREL',
                    'LICENCAUSO.NMFANTLUSO AS LOCALSERVCDAREL',
                    'SERVPREST.NRPROPOSFAS',
                    'SERVPREST.NRVERPROPFAS',
                    "PROPOSTAFAS.NRPROPOSFAS || ' - ' || PROPOSTAFAS.DSREFEPROPOSFAS AS DSREFEPROPOSFASCDAREL",
                    'PROJETO.CDPROJETO',
                    'TIPOSERV.CDTIPOSERV',
                    'PROJETO.NMPROJETO AS NMPROJETOCDAREL',
                    'TIPOSERV.NMTIPOSERV AS NMTIPOSERVCDAREL',
                    "TO_CHAR(SERVPREST.DTENTRADA, 'HH24:MI') AS DTENTRADACDAREL",
                    /* 'SERVPREST.HRAUSENCIAALMOCO AS HRAUSENCIAALMOCOCDAREL',*/
                    "DECODE(SERVPREST.HRAUSENCIAALMOCO, NULL, '00:00', SUBSTR(SERVPREST.HRAUSENCIAALMOCO,1,2)||':'||SUBSTR(SERVPREST.HRAUSENCIAALMOCO,-2,2)) AS HRAUSENCIAALMOCOCDAREL",
                    "TO_CHAR(SERVPREST.DTSAIDA, 'HH24:MI') AS DTSAIDACDAREL",
                    /* 'SERVPREST.QTHRTOTAL AS QTHRTOTALCDAREL', */
                    "DECODE(SERVPREST.QTHRTOTAL, NULL, '00:00', SUBSTR(SERVPREST.QTHRTOTAL,1,2)||':'||SUBSTR(SERVPREST.QTHRTOTAL,-2,2)) AS QTHRTOTALCDAREL",
                    /* 'SERVPREST.QTHRDESLOCAMENTOFAT AS QTHRDESLOCAMENTOFATCDAREL', */
                    "DECODE(SERVPREST.QTHRDESLOCAMENTOFAT, NULL, '00:00', SUBSTR(SERVPREST.QTHRDESLOCAMENTOFAT,1,2)||':'||SUBSTR(SERVPREST.QTHRDESLOCAMENTOFAT,-2,2)) AS QTHRDESLOCAMENTOFATCDAREL",
                    /* 'SERVPREST.QTHRATIVIDADEFAT AS QTHRATIVIDADEFATCDAREL', */
                    "DECODE(SERVPREST.QTHRATIVIDADEFAT, NULL, '00:00', SUBSTR(SERVPREST.QTHRATIVIDADEFAT,1,2)||':'||SUBSTR(SERVPREST.QTHRATIVIDADEFAT,-2,2)) AS QTHRATIVIDADEFATCDAREL",
                    'HORADEC_HORACHAR(HORACHAR_HORADEC(QTHRTOTAL) - HORACHAR_HORADEC(QTHRATIVIDADEFAT)) AS DSJUSTHRNAOFATURADACDAREL',
                    'SERVPRESTDOCASSINA.IDSITASSINATURA AS SITUACAOASSCDAREL',
                    'SERVPRESTDOCASSINA.DTASSINATURA AS DATAASSCDAREL',
                    "CASE WHEN SERVPREST.IDSERVPREST = 'FALTA_NAO_JUSTIFICADA' THEN 'S' ELSE NULL END AS IDFALTACDAREL",
                    'DESCSERVPRES.DSSERVPREST AS DESCRICAOATVCDAREL',
                    'OPERADOR.NRPARCNEGOCIO',
                    'PERFACESSPARC.NRPERFACESSPARC',
                    'PERFILACESSO.NRPERFILACESSO',
                    'PERFILACESSO.NMPERFILACESSO'
                ])
                ->from('SERVPREST', 'SERVPREST','OPERADOR', 'OPER')
                ->leftJoin('SERVPREST', 'CLIENTE', 'CLIENTE', 'SERVPREST.CDCLIENTE = CLIENTE.CDCLIENTE')
                ->leftJoin('SERVPREST', 'FUNCIONARIO', 'FUNCIONARIO', 'FUNCIONARIO.CDFUNC = SERVPREST.CDFUNC')
                ->leftJoin('SERVPREST', 'TIPOSERV', 'TIPOSERV', 'TIPOSERV.CDTIPOSERV = SERVPREST.CDTIPOSERV')
                ->leftJoin('SERVPREST', 'LICENCAUSO', 'LICENCAUSO', 'LICENCAUSO.CDCLIENTE = SERVPREST.CDCLIENTE AND LICENCAUSO.NRLICENCAUSO = SERVPREST.NRLICENCAUSO')
                ->leftJoin('SERVPREST', 'PROPOSTAFAS', 'PROPOSTAFAS', 'PROPOSTAFAS.NRPROPOSFAS = SERVPREST.NRPROPOSFAS')
                ->leftJoin('SERVPREST', 'PROJETO', 'PROJETO', 'SERVPREST.CDPROJETO = PROJETO.CDPROJETO')
                ->leftJoin('SERVPREST', 'DESCSERVPRES', 'DESCSERVPRES', 'DESCSERVPRES.CDFUNC = SERVPREST.CDFUNC AND DESCSERVPRES.CDCLIENTE = SERVPREST.CDCLIENTE AND DESCSERVPRES.DTSERVPREST = SERVPREST.DTSERVPREST AND DESCSERVPRES.CDEMPRESA = CLIENTE.CDEMPRESA AND DESCSERVPRES.NRSERVPREST = SERVPREST.NRSERVPREST')
                ->leftJoin('SERVPREST', 'SERVPRESTDOC', 'SERVPRESTDOC', 'SERVPREST.CDEMPRESA = SERVPRESTDOC.CDEMPRESA AND SERVPREST.CDFUNC = SERVPRESTDOC.CDFUNC AND SERVPREST.CDCLIENTE = SERVPRESTDOC.CDCLIENTE AND SERVPREST.DTSERVPREST = SERVPRESTDOC.DTSERVPREST AND SERVPREST.NRSERVPREST = SERVPRESTDOC.NRSERVPREST')
                ->leftJoin('SERVPREST', 'SERVPRESTDOCASSINA', 'SERVPRESTDOCASSINA', 'SERVPRESTDOC.NRSERVPRESTDOC = SERVPRESTDOCASSINA.NRSERVPRESTDOC')
                ->leftJoin('FUNCIONARIO', 'OPERADOR', 'OPERADOR', 'FUNCIONARIO.CDFUNC = OPERADOR.CDFUNC')
                ->leftJoin('OPERADOR', 'PARCNEGOCIO', 'PARCNEGOCIO', 'OPERADOR.NRPARCNEGOCIO = PARCNEGOCIO.NRPARCNEGOCIO')
                ->leftJoin('PARCNEGOCIO', 'PERFACESSPARC', 'PERFACESSPARC', 'PERFACESSPARC.NRPARCNEGOCIO = PARCNEGOCIO.NRPARCNEGOCIO')
                ->leftJoin('PERFACESSPARC', 'PERFILACESSO', 'PERFILACESSO', 'PERFACESSPARC.NRPERFILACESSO = PERFILACESSO.NRPERFILACESSO')
                ->where($queryBuilder->expr()->isNotNull('PROPOSTAFAS.DSREFEPROPOSFAS'))
                ->andWhere($queryBuilder->expr()->neq('PROPOSTAFAS.DSREFEPROPOSFAS', "'.'"));
              

            $result = $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $queryBuilder->getSQL(), $params);
            return $result;
        } catch (\Exception $e) {
            echo 'Ocorreu uma exceção: ' . $e->getMessage();
            throw $e;
        }
    }

    public function getClienteRelCda($parametros): array
    {
        $nrorg = $parametros['NRORG'];

        $queryBuilder  = $this->connection->createQueryBuilder();

        $queryBuilder
            ->select("DISTINCT CLIENTE.CDCLIENTE, CLIENTE.NMFANTCLIE AS CLIENTECDAREL, 
                      CLIENTE.NRORG, CLIENTE.IDATIVO, CLIENTE.NMFANTCLIE || DECODE(CLIENTE.IDCLIEATIV, 'N', ' - INATIVO') AS DISPLAYCLIENTE")
            ->from('CLIENTE', 'CLIENTE')
            ->where(
                $queryBuilder->expr()->and(
                    $queryBuilder->expr()->eq('CLIENTE.NRORG', ':nrorg')
                )
            );
        
            $queryBuilder->setParameter('nrorg', $nrorg);
        $result = $queryBuilder->execute()->fetchAllAssociative();
        return $this->buildResponseData($result);
    }

    private function applyDateFilter($queryBuilder, $dateField, $dateValuesMin, $dateValuesMax): void
    {
        $dateTimeMin = \DateTime::createFromFormat('d/m/Y', $dateValuesMin);

        if ($dateTimeMin !== false && $dateValuesMax === null) {
            $formattedDateMin = $dateTimeMin->format('Y-m-d');
            $queryBuilder->andWhere(
                $queryBuilder->expr()->eq("TRUNC($dateField)", 'TO_DATE(:dateValuesMin, \'YYYY-MM-DD\')')
            )->setParameter('dateValuesMin', $formattedDateMin);
        }

        if ($dateValuesMax !== null) {
            $dateTimeMax = \DateTime::createFromFormat('d/m/Y', $dateValuesMax);

            if ($dateTimeMax !== false) {
                $formattedDateMin = $dateTimeMin->format('Y-m-d');
                $formattedDateMax = $dateTimeMax->format('Y-m-d');

                $queryBuilder->andWhere(
                    $queryBuilder->expr()->andX(
                        $queryBuilder->expr()->gte("TRUNC($dateField)", 'TO_DATE(:dateValuesMin, \'YYYY-MM-DD\')'),
                        $queryBuilder->expr()->lte("TRUNC($dateField)", 'TO_DATE(:dateValuesMax, \'YYYY-MM-DD\')')
                    )
                )->setParameters([
                    'dateValuesMin' => $formattedDateMin,
                    'dateValuesMax' => $formattedDateMax
                ]);
            }
        }
    }

    public function getPropostaCdaRel($parametros)
    {
        $nrorg = $parametros["NRORG"];
        // $dateValuesMin = $parametros['DTSERVPREST_MIN'] ?? null;
        // $dateValuesMax = $parametros['DTSERVPREST_MAX'] ?? null;
        $cdClient = isset($parametros["CDCLIENTE"]) ? $parametros["CDCLIENTE"] : null;

        $queryBuilder  = $this->connection->createQueryBuilder();


        $queryBuilder
        ->select(
            'DISTINCT PROPOSTA.DSREFEPROPOSFAS AS DSREFEPROPOSFASCDAREL',
            'PROPOSTA.NRPROPOSFAS',
            'PROPOSTA.CDCLIENTE AS CLIENTECDAREL',
            'SERVPREST.CDFUNC',
            'SERVPREST.DTSERVPREST'
        )
        ->from('PROPOSTAFAS', 'PROPOSTA')
        ->leftJoin(
            'PROPOSTA',
            'SERVPREST',
            'SERVPREST',
            'SERVPREST.NRVERPROPFAS = PROPOSTA.NRVERPROPFAS AND SERVPREST.NRPROPOSFAS = PROPOSTA.NRPROPOSFAS'
        )
        ->where(
            $queryBuilder->expr()->and(
                $queryBuilder->expr()->isNotNull('PROPOSTA.DSREFEPROPOSFAS'),
                $queryBuilder->expr()->neq('PROPOSTA.DSREFEPROPOSFAS', "'.'")
            )
        );
    
    
    


        $queryBuilder->setParameter('nrorg', $nrorg);
        // $this->applyDateFilter($queryBuilder, 'SERVPREST.DTSERVPREST', $dateValuesMin, $dateValuesMax);
        $this->verifyClientFilter($queryBuilder, 'PROPOSTA.CDCLIENTE', $cdClient);
        $result = $queryBuilder->execute()->fetchAllAssociative();
        return $this->buildResponseData($result);
    }

    private function verifyClientFilter($queryBuilder, $cdClientField, $cdClient): void
    {
        if (!empty($cdClient)) {
            $queryBuilder->andWhere(
                $queryBuilder->expr()->in($cdClientField, ':CDCLIENTE')
            )->setParameter('CDCLIENTE', $cdClient, Connection::PARAM_STR_ARRAY);
        }
    }

    public function getProjetoCdaRel($parametros)
    {

        $nrorg = $parametros["NRORG"];
        $nrProposFas = isset($parametros["NRPROPOSFAS"]) ? $parametros["NRPROPOSFAS"] : null;
        $queryBuilder  = $this->connection->createQueryBuilder();

        $queryBuilder
            ->select('PROJETO.NMPROJETO AS NMPROJETOCDAREL, PROJETO.IDATIVO, PROJETO.NRORG, PROJETO.NRPROPOSFAS, PROJETO.CDCLIENTE, PROJETO.CDPROJETO')
            ->from('PROJETO', 'PROJETO')
            ->leftJoin('PROJETO', 'SERVPREST', 'SERVPREST', 'SERVPREST.NRVERPROPFAS = PROJETO.NRVERPROPFAS AND SERVPREST.NRPROPOSFAS = PROJETO.NRPROPOSFAS')
            ->where($queryBuilder->expr()->eq('PROJETO.IDATIVO', "'S'"))
            ->andWhere($queryBuilder->expr()->eq('PROJETO.NRORG', ':NRORG'))
            ->andWhere($queryBuilder->expr()->in('PROJETO.NRPROPOSFAS', ':NRPROPOSFAS'));

        $queryBuilder->setParameter('NRORG', $nrorg);
        $queryBuilder->setParameter('NRPROPOSFAS', $nrProposFas, Connection::PARAM_STR_ARRAY);
        $result = $queryBuilder->execute()->fetchAllAssociative();
        return $this->buildResponseData($result);
    }

    public function getTypeOfActivityCdaRel($parametros)
    {
        $nrorg = $parametros["NRORG"];
        // $dateValuesMin = $parametros['DTSERVPREST_MIN'] ?? null;
        // $dateValuesMax = $parametros['DTSERVPREST_MAX'] ?? null;
        $cdClient = isset($parametros["CDCLIENTE"]) ? $parametros["CDCLIENTE"] : null;
        $nrProposFas = isset($parametros["NRPROPOSFAS"]) ? $parametros["NRPROPOSFAS"] : null;

        $queryBuilder  = $this->connection->createQueryBuilder();

        $queryBuilder
            ->select('TIPOSERV.NMTIPOSERV AS NMTIPOSERVCDAREL, TIPOSERV.CDTIPOSERV, CONTRATOSERV.CDCLIENTE, CONTRATOSERV.NRPROPOSFAS, CONTRATOSERV.NRVERPROPFAS, 
        CONTRATOSERV.NRSERVCONT, CONTRATOSERV.QTSERVCONT, AGENDASERV.NRORG, AGENDASERV.CDFUNC')
            ->from('TIPOSERV', 'TIPOSERV')
            ->leftJoin('TIPOSERV', 'CONTRATOSERV', 'CONTRATOSERV', 'CONTRATOSERV.CDTIPOSERV = TIPOSERV.CDTIPOSERV AND CONTRATOSERV.NRORG = TIPOSERV.NRORG')
            ->leftJoin('TIPOSERV', 'AGENDASERV', 'AGENDASERV', 'AGENDASERV.CDTIPOSERV = TIPOSERV.CDTIPOSERV AND AGENDASERV.NRORG = CONTRATOSERV.NRORG AND AGENDASERV.NRVERPROPFAS = CONTRATOSERV.NRVERPROPFAS
                    AND AGENDASERV.CDTIPOSERV = CONTRATOSERV.CDTIPOSERV AND AGENDASERV.NRSERVCONT = CONTRATOSERV.NRSERVCONT AND AGENDASERV.NRPROPOSFAS = CONTRATOSERV.NRPROPOSFAS')
            ->where(
                $queryBuilder->expr()->and(
                    $queryBuilder->expr()->eq('CONTRATOSERV.IDATIVO', "'S'"),
                    $queryBuilder->expr()->eq('CONTRATOSERV.NRORG', ':NRORG')
                )
            )
            ->orderBy('TIPOSERV.CDTIPOSERV');

        // $this->applyDateFilterForTypeActivity($queryBuilder, $dateValuesMin, $dateValuesMax);
        $queryBuilder->setParameter('NRORG', $nrorg);
        $this->verifyClientFilter($queryBuilder, 'CONTRATOSERV.CDCLIENTE', $cdClient);
        $this->verifyProposalFilter($queryBuilder, 'CONTRATOSERV.NRPROPOSFAS', $nrProposFas);
        $result = $queryBuilder->execute()->fetchAllAssociative();
        return $this->buildResponseData($result);
    }

    private function verifyProposalFilter($queryBuilder, $nrProposFasField, $nrProposFas): void
    {
        if (!empty($nrProposFas)) {
            $queryBuilder->andWhere(
                $queryBuilder->expr()->in($nrProposFasField, ':NRPROPOSFAS')
            )->setParameter('NRPROPOSFAS', $nrProposFas, Connection::PARAM_STR_ARRAY);
        }
    }

    private function applyDateFilterForTypeActivity($queryBuilder, $dateValuesMin, $dateValuesMax): void
    {
        if (!empty($dateValuesMin) && $dateValuesMax === null) {
            $dateTimeMin = \DateTime::createFromFormat('d/m/Y', $dateValuesMin);
            $formattedDateMin = $dateTimeMin->format('Y-m-d');

            $queryBuilder->andWhere(
                $queryBuilder->expr()->andX(
                    $queryBuilder->expr()->gte("AGENDASERV.DTINITRASERV", "TO_DATE(:dateValuesMin, 'YYYY-MM-DD')"),
                    $queryBuilder->expr()->lte("AGENDASERV.DTFINTRASERV", "TO_DATE(:dateValuesMin, 'YYYY-MM-DD')")
                )
            )->setParameter('dateValuesMin', $formattedDateMin);
        } else if (!empty($dateValuesMin) && !empty($dateValuesMax)) {
            $dateTimeMin = \DateTime::createFromFormat('d/m/Y', $dateValuesMin);
            $dateTimeMax = \DateTime::createFromFormat('d/m/Y', $dateValuesMax);
            $formattedDateMin = $dateTimeMin->format('Y-m-d');
            $formattedDateMax = $dateTimeMax->format('Y-m-d');

            $queryBuilder->andWhere(
                $queryBuilder->expr()->andX(
                    $queryBuilder->expr()->gte("AGENDASERV.DTINITRASERV", "TO_DATE(:dateValuesMin, 'YYYY-MM-DD')"),
                    $queryBuilder->expr()->lte("AGENDASERV.DTFINTRASERV", "TO_DATE(:dateValuesMax, 'YYYY-MM-DD')")
                )
            )->setParameters([
                'dateValuesMin' => $formattedDateMin,
                'dateValuesMax' => $formattedDateMax
            ]);
        }
    }

    public function getFuncRelCda($parametros)
    {

        $nrorg = $parametros["NRORG"];
        $nrperfilacesso = isset($parametros["NRPERFILACESSO"]) ? $parametros["NRPERFILACESSO"] : null;
        $queryBuilder  = $this->connection->createQueryBuilder();

        $queryBuilder
            ->select('DISTINCT FUNCIONARIO.CDFUNC AS CDOPERADORCDAREL, FUNCIONARIO.NMFUNC AS NMOPERADORCDAREL, PERFILACESSO.NMPERFILACESSO, PERFILACESSO.NRPERFILACESSO' )
            ->from('FUNCIONARIO', 'FUNCIONARIO')
            ->leftJoin('FUNCIONARIO', 'OPERADOR', 'OPERADOR', 'OPERADOR.CDFUNC = FUNCIONARIO.CDFUNC')
            ->leftJoin('OPERADOR', 'PARCNEGOCIO', 'PARCNEGOCIO', 'OPERADOR.NRPARCNEGOCIO = PARCNEGOCIO.NRPARCNEGOCIO')
            ->leftJoin('PARCNEGOCIO', 'PERFACESSPARC', 'PERFACESSPARC', 'PERFACESSPARC.NRPARCNEGOCIO = PARCNEGOCIO.NRPARCNEGOCIO')
            ->leftJoin('PERFACESSPARC', 'PERFILACESSO', 'PERFILACESSO', 'PERFACESSPARC.NRPERFILACESSO = PERFILACESSO.NRPERFILACESSO')
            ->where($queryBuilder->expr()->eq('FUNCIONARIO.IDATIVO', "'S'"))
            ->andWhere($queryBuilder->expr()->eq('FUNCIONARIO.NRORG', ':NRORG'));

        $queryBuilder->setParameter('NRORG', $nrorg);
        $queryBuilder->setParameter('NRPERFILACESSO', $nrperfilacesso);
        $this->verifyPerfilFilter($queryBuilder, 'PERFILACESSO.NRPERFILACESSO', $nrperfilacesso);
        $result = $queryBuilder->execute()->fetchAllAssociative();
        return $this->buildResponseData($result);
    }

    private function verifyPerfilFilter($queryBuilder, $nrperfilacessoField, $nrperfilacesso): void
    {
        if (!empty($nrperfilacesso)) {
            $queryBuilder->andWhere(
                $queryBuilder->expr()->in($nrperfilacessoField, ':NRPERFILACESSO')
            )->setParameter('NRPERFILACESSO', $nrperfilacesso, Connection::PARAM_STR_ARRAY);
        }
    }

    public function getPerfAcesRelCda($parametros)
    {

        $nrorg = $parametros["NRORG"];
        $queryBuilder  = $this->connection->createQueryBuilder();

        $queryBuilder
        ->select('NRPERFILACESSO, NMPERFILACESSO')
        ->from('PERFILACESSO', 'PERF')
        ->where($queryBuilder->expr()->eq('PERF.IDATIVO', "'S'"))
        ->orderBy('NMPERFILACESSO', 'ASC');

        $queryBuilder->setParameter('NRORG', $nrorg);
        $result = $queryBuilder->execute()->fetchAllAssociative();
        return $this->buildResponseData($result);
    }
    
}
