<?php

namespace SM\Service;

use SM\Factory\HabilidadePessoalFactory;
use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\NewCode;
use Doctrine\ORM\EntityManager;

class HabilidadePessoalService
{
    protected $connection;
    protected $habilidadePessoalFactory;
    protected $environment;
    protected $newCode;

    public function __construct(HabilidadePessoalFactory $habilidadePessoalFactory, Environment $environment, NewCode $newCode, EntityManager $entityManager)
    {
        $this->habilidadePessoalFactory     = $habilidadePessoalFactory;
        $this->environment                  = $environment;
        $this->newCode                      = $newCode;
        $this->connection                   = $entityManager->getConnection();
    }

    public function getHabilidadePessoal()
    {
       return $this->habilidadePessoalFactory->getHabilidadePessoal();
    }
    
    public function adicionarHabilidadePessoal($params)
    {
        
        try {
            $nrorg = $this->environment->getNrOrg();
            $cdoperador = $this->environment->getCdOperador();
            $novoCodigo = $this->newCode->getCodeByTableName("FCT_HABILITSERV", $nrorg, 1, 10, true);
            $dataParams = array(
                "CDHABILITSERV"         => $novoCodigo,
                "NMHABILITSERV"         => $params["NMHABILITSERV"],
                "NRORG"                 => $nrorg,
                "CDOPERINCLUSAO"          => $cdoperador
            );
            $this->habilidadePessoalFactory->adicionarHabilidadePessoal($dataParams);
        } catch (\Exception $e) {
            throw $e;
        }

    }

    public function getHabPessoalTabPessoas($params)
    {
       return $this->habilidadePessoalFactory->getHabPessoalTabPessoas($params);
    }

    public function deleteHabilidadePessoal($params)
    {
        $this->connection->beginTransaction();
        try {
            foreach ($params as $habilidadePessoal) {
                $dataParam = array(
                    "CDHABILITSERV" => $habilidadePessoal["CDHABILITSERV"],
                );
                $this->habilidadePessoalFactory->deleteHabilidadePessoal($dataParam);
            }
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function saveTabHabilidadePessoal($params)
    {
        $cdoperador = $this->environment->getCdOperador();
        $nrorg = $this->environment->getNrOrg();
        $this->connection->beginTransaction();

        try {
            $dataParam = array(
                "CDHABILITSERV"     => $params["CDHABILITSERV"],
                "NMHABILITSERV"     => $params["NMHABILITSERV"],                  
                "NRORG"             => $nrorg,
                "IDATIVO"           => "S",
                "NRORGULTATU"       => $nrorg,
                "CDOPERULTATU"      => $cdoperador
            );
            $this->habilidadePessoalFactory->saveTabHabilidadePessoal($dataParam);
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function getHabPessoalPessoas()
    {
        $dataParam = array(
            "CDTIPOPARCEIRO"    => "PESSOA",
            "IDATIVO"           => "S",
        );
       return $this->habilidadePessoalFactory->getHabPessoalPessoas($dataParam);
    }

    public function saveModalAddPessoa($params)
    {
        $cdoperador = $this->environment->getCdOperador();
        $nrorg = $this->environment->getNrOrg();
        $this->connection->beginTransaction();

        try {
            foreach ($params["PESSOAS"] as $pessoa) {               
                $novoCodigo = $this->newCode->getCodeByTableName("FCT_RELACHABILIPESSOA", $nrorg, 1, 10, true);
                $dataParam = array(
                    "NRRELACHABILIPESSOA"   => $novoCodigo,
                    "CDHABILITSERV"         => $params["CDHABILITSERV"],
                    "NRPARCNEGOCIO"         => $pessoa["NRPARCNEGOCIO"],
                    "NRORG"                 => $nrorg,
                    "CDOPERINCLUSAO"        => $cdoperador,
                    "IDATIVO"               => "S",
                );
                $this->habilidadePessoalFactory->saveModalAddPessoa($dataParam);
            }
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function deletePessoas($params)
    {
        $this->connection->beginTransaction();
        try {
            foreach ($params as $habilidadePessoal) {
                $dataParam = array(
                    "NRRELACHABILIPESSOA" => $habilidadePessoal["NRRELACHABILIPESSOA"],
                );
                $this->habilidadePessoalFactory->deletePessoas($dataParam);
            }
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }
}